        .module crt0
        .globl  _debug_swbreak
        .globl  _debug_nmi
        .globl  _main

	.area	_HEADER (ABS)
	;; Reset vector
	.org 	0
_start:
	jp	init

	.org	0x08
    jp      _debug_swbreak
	.org    0x10
	ei
	reti
	.org    0x18
	ei
	reti
	.org    0x20
	ei
	reti
	.org    0x28
	ei
	reti
	.org    0x30
	ei
	reti
	.org    0x38
	ei
	reti
	
;;;  NMI will be used for the 'bash' button
;;;  If the user bashes this button, control will be given back to the
;;;  monitor in order to resync with the debugger
	.org    0x66
    jp _debug_nmi

	.org 0x100

init:
;; Stack at the top of memory.
;	ld	sp, #0xffff
	ld	sp, #0xff00

    ;; Initialise global variables
    call gsinit
	ld hl,#5
	push hl
	call _debug_exception
	call _main
	jp _exit

	.org 0x120  ; the start address before relocation
run:
	di
	LD	BC,#0x3FFF
	in	a,(c)					; disable CSS Electronics 128KB ROM board if present
	call SET_AY_PORTS+0x8000	; called function must not call or jump anywhere, only ret
	LD A,(0x5b5c)      			; Previous value of bank switching port from OS variable
	AND	#0xf8					; bank 0
	LD	BC,#0x7ffd				; bank switching port address
;	LD	(0x5b5c),A				; Reporting new bank to OS - not needed as whole memory gets mixed up
	OUT	(C),A					; actual bank switching

	ld hl,#0x8000				; copy from bank 2 (where we are running now)
	ld de,#0xc000				; to bank 0
	ld bc,#0x1000				; up to 4 kilobytes
	ldir						; do the copy

	ld a,#1						; special mode, all-ram, banks 0 1 2 3,
								; bank 0 at 0x0000 bank 2 should be still at the same place (0x8000)
	LD BC,#0x1ffd				; special bank switching port addresss
	OUT (C),A					; actual bank switching
								; bootstrap done, we can start regularly from 0x0000
	rst 0						; start from 0 in the bank 0


	;; Ordering of segments for the linker.
	.area	_HOME
	.area	_CODE
	.area	_INITIALIZER
    .area   _GSINIT
    .area   _GSFINAL

	.area	_DATA
	.area	_INITIALIZED
	.area	_BSEG
    .area   _BSS
    .area   _HEAP

    .area   _CODE
__clock::
	ld	a,#2
    rst 0x08
	ret

_exit::
	;; Exit - special code to the emulator
	ld	a,#0
    rst 0x08
1$:
	halt
	jr	1$

	.area   _GSINIT
gsinit::
	ld	bc, #l__INITIALIZER
	ld	a, b
	or	a, c
	jr	Z, gsinit_next
	ld	de, #s__INITIALIZED
	ld	hl, #s__INITIALIZER
	ldir
gsinit_next:

	.area   _GSFINAL
	ret
